#!/bin/sh

# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Raymond Mancy<rmancy@redhat.com> 

# source the test script helpers
. /usr/bin/rhts-environment.sh

# Assume the test will fail.
result=FAIL

# Helper functions
# control where to log debug messages to:
# devnull = 1 : log to /dev/null
# devnull = 0 : log to file specified in ${DEBUGLOG}
devnull=0

# Create debug log
DEBUGLOG=`mktemp -p /mnt/testarea -t DeBug.XXXXXX`

# locking to avoid races
lck=$OUTPUTDIR/$(basename $0).lck

# Log a message to the ${DEBUGLOG} or to /dev/null
function DeBug ()
{
    local msg="$1"
    local timestamp=$(date '+%F %T')
    if [ "$devnull" = "0" ];then
	lockfile -r 1 $lck
	if [ "$?" = "0" ];then
	    echo -n "${timestamp}: " >>$DEBUGLOG 2>&1
	    echo "${msg}" >>$DEBUGLOG 2>&1
	    rm -f $lck >/dev/null 2>&1
	fi
    else
	echo "${msg}" >/dev/null 2>&1
    fi
}

function SubmitLog ()
{
    LOG=$1
    rhts_submit_log -S $RESULT_SERVER -T $TESTID -l $LOG
}

function result_fail()
{
    echo "***** End of runtest.sh *****" | tee -a $OUTPUTFILE
    report_result $TEST FAIL 1
    SubmitLog $DEBUGLOG
    CleanUp
    exit 0
}

function result_pass ()
{
    echo "***** End of runtest.sh *****" | tee -a $OUTPUTFILE
    report_result $TEST PASS 0
    CleanUp
    exit 0
}

#Function below will report if a prev command failed
function estatus_fail()
{
    if [ "$?" -ne "0" ];then
	DeBug "$1 Failed"
	echo "***** "$1" Failed: *****" | tee -a $OUTPUTFILE
        rhts-sync-set -s ABORT
	result_fail
    fi
}

function install_selenium_python()
{
    easy_install selenium
}

function install_selenium_server()
{
    wget http://selenium.googlecode.com/files/selenium-remote-control-$RC_VERSION.zip 
    mkdir -p $SELENIUM_RC_PATH
    mv -f selenium-remote-control-$RC_VERSION.zip $SELENIUM_RC_PATH
    cd $SELENIUM_RC_PATH
    unzip -o selenium-remote-control-$RC_VERSION.zip    
}

function do_prereqs()
{
    yum install -y $PACKAGES  
    estatus_fail "**** Yum Install of $PACKAGES Failed ****"
    Xvfb :4 -fp /usr/share/X11/fonts/misc -screen 0 1024x768x24 2>/dev/null > /dev/null &
}




function run_selenium_server()
{
    env DISPLAY=:4 java -jar $SELENIUM_RC_PATH/selenium-server-$RC_VERSION/selenium-server.jar 2>/dev/null >/dev/null &
    estatus_fail "**** Failed to start selenium server ****"
}

function run_test_case () {

	python $1 >$OUTPUTFILE

	if [ $? -eq 0 ]
	then
		report_result ${TEST}/$i/$i PASS ${SCORE}
                result="PASS"
	else
		report_result ${TEST}/$i/$i FAIL ${SCORE}	
	fi
}


SELENIUM_RC_PATH=/usr/lib/selenium
RC_VERSION=1.0.3
PACKAGES="firefox Xvfb java xorg-x11-fonts-*"
install_selenium_python
install_selenium_server
do_prereqs
run_selenium_server

{
	#log "[`date +%H:%M:%S`] Start Test "
	for i in `ls $TESTPATH/*.py`
	do 
		file_name=$i
		#log "Running Selenium tests $i"
		run_test_case "$i"
	done

         #log "Overall result: ---$result"

} 2>&1 | tee ${OUTPUTFILE}


#rhts-run-simple-test $TEST ./testing.py




